###############################################################################
#                                                                             #
#                Adding license pack  for users for Office 365                #
#                                    V2.81                                    #
#                       (C)2006-2014 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)]: $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Adding license pack for users for Office 365 $($Version)" -foregroundcolor "white"
write-host "Configurating regional options" -foregroundcolor "white"
write-host "(C) 2006-2014 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
if ($ProxyEnabled)
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -SessionOption $PSSessionOption -ErrorAction SilentlyContinue
   }
   else
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
   }
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function user_exists
{
Param([string] $email)
$error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -ArgumentList $email {Param($id) Get-User -Identity $id}
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
## Install MSOnline Cmdlets 
function Install-MSOnlineCmdlets()
{
$Res=$false
$DotNET35Path="HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v3.5"
$DotNet35Present=((Test-Path $DotNET35Path) -and ((Get-ItemProperty $DotNET35Path).Install -eq "1"))
if ($DotNet35Present)
   {
   # Trying to import module
   if (-not(Get-Module -Name "MSOnline")){Get-Module -ListAvailable | Where-Object {$_.name -eq "MSOnline"} | Import-Module}
   # Module is not present
   if (-not(Get-Module -Name "MSOnline"))
      {
      Write-Host "MSOnline Powershell module is not present !" -Foregroundcolor "Red"
      $wc=New-Object System.Net.WebClient
      # Sign-in Assistant
      $SigninAssistantPath="HKLM:\SOFTWARE\Microsoft\MSOIdentityCRL"
      $SigninAssistantPresent=((Test-Path $SigninAssistantPath) -and ((Get-ItemProperty $SigninAssistantPath).TargetDir -ne $null))
      if (!$SigninAssistantPresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            #{$Binary="http://go.microsoft.com/fwlink/?linkid=236300"}
            #{$Binary="http://go.microsoft.com/fwlink/?LinkId=221500"}
            {$Binary="http://download.microsoft.com/download/E/1/7/E17BB123-19FA-4B91-83DB-903AD0DB5DB8/msoidcli_64.msi"}
            else
            #{$Binary="http://go.microsoft.com/fwlink/?linkid=236299"}
            #{$Binary="http://go.microsoft.com/fwlink/?LinkId=221498"}
            {$Binary="http://download.microsoft.com/download/E/1/7/E17BB123-19FA-4B91-83DB-903AD0DB5DB8/msoidcli_32.msi"}
         # $MyCulture="0x{0:X}" -f(Get-Culture).LCID
         # $Source=$Binary+"&clcid="+$Myculture
         $Source=$Binary
         $Destination=$env:TEMP+"\_SigninAssistant_.msi"
         Write-Host "Trying to download ""Microsoft Online Sign-in Assistant"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Sign-in Assistant installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Sign-in Assistant, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            } 
         $FileExists=Test-Path $Destination
         If ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # MSOnline module
      $MSOnlinePath="HKLM:\SOFTWARE\Microsoft\MSOnlinePowerShell"
      $MSOnlinePresent=((Test-Path $MSOnlinePath) -and ((Get-ItemProperty $MSOnlinePath).InstallPath -ne $null))
      if (!$MSOnlinePresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture -eq "64 bits")
            {
            # 64 bits
            if ((Get-WmiObject Win32_OperatingSystem).Version -lt "6")
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x64-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               $Binary="http://go.microsoft.com/fwlink/?linkid=236297"
               $Source=$Binary+"&clcid="+$MyCulture
               }
            }
            else
            {
            # 32 bits
            if ((Get-WmiObject Win32_OperatingSystem).Version -lt "6")
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               $Binary="http://go.microsoft.com/fwlink/?linkid=236298"
               $Source=$Binary+"&clcid="+$MyCulture
               }
            }
         $Destination=$env:TEMP+"\_MSOnline_.msi"
         write-host "Trying to download ""Microsoft Online Powershell Module"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Powershell Module installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Powershell Module, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            }
         $FileExists=Test-Path $Destination
         if ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # Import MSOnline Module
      $Mod=Import-Module "MSOnline" -PassThru
      if ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="MSOnline Powershell module is still not present !"
         Write-Host $S -Foregroundcolor "Red"
         Log_Message "ERROR" $S
         Write-Host "Press any key ..."
         Read-Host
         }
         else
         {
         $S="MSOnline Powershell module is now present and loaded !"
         Write-Host $S -Foregroundcolor "Green"
         Log_Message "OK" $S
         $Res=$true
         }
      }
      else
      {
      Write-Host "MSOnline Powershell module is present" -Foregroundcolor "Green"
      $Res=$true
      }
  }
  else
  {
   $S="Please, install Framework .NET V3.5 first !"
   Write-Host $S -ForegroundColor "Red"
   Log_Message "ERROR" $S
   Write-Host "Press any key ..."
   Read-Host
  }
return $Res
}
###############################################################################
function UserMSolExist
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function UserHasAtLeastOneLicense
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
return (![string]::IsNullOrEmpty($User.Licenses))
}
###############################################################################
function UserHasLicense($email,$LicensePack)
{
# Param([string] $email,$LicensePack)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
$Res=$false
If ($User.Licenses.count -gt 0)
   {
   ForEach ($License In $User.Licenses)
           {
           If ($License.AccountSku.SkuPartNumber.ToLower() -eq $LicensePack.ToLower())
              {
              $Res=$true;
              }
           }
   }
return $Res
}
###############################################################################
function UserLicenses
{
Param([string] $email)
$User=Get-MSolUser -UserPrincipalName $email -ErrorAction SilentlyContinue
$Res=""
If ($User.Licenses.count -gt 0)
   {
   ForEach ($License In $User.Licenses)
           {
           $Lic="$($License.AccountSku.AccountName):$($License.AccountSku.SkuPartNumber)"
           $Res=$Res+" "+$Lic
           }
   }
return $Res.Trim()
}
###############################################################################
function LicensePackDomain
{
$sku=Get-MsolAccountSku
if ($sku.count -gt 0)
   {
   return $sku[0].AccountSkuId.Split(":")[0]
   }
   else
   {
   return ""
   }
}
###############################################################################
function connect_msol
{
Param($Cred)
Connect-MsolService -Credential $Cred	-ErrorAction SilentlyContinue
}
###############################################################################
function addlicensepack_user
{
Param($UserDisplayName,$UserEmail,$LicensePack,$UsageLocation)
$error.Clear()
Set-MSolUser -UserPrincipalName $UserEmail -UsageLocation $UsageLocation -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully add location ""$($UsageLocation)"" for ""$($UserDisplayName)"" ""($($UserEmail))"""
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Adding location ""$($UserLocation)"" for ""$($UserDisplayName)"" ""($($UserEmail))"". Error = $($error[0])"
    log_message "ERROR" $S
    }
Set-MSolUserLicense -UserPrincipalName $UserEmail -AddLicenses $LicensePack -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
   $S="Successfully add license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"""
   log_message "OK" $S
   }
   else
   {
   $global:errors_count++
   $S="Adding license pack ""$($LicensePack)"" for ""$($UserDisplayName)"" ""($($UserEmail))"". Error = $($error[0])"
   log_message "ERROR" $S
   }
}
###############################################################################
function regional_configuration_user
{
Param($UserDisplayName,$UserEmail,$Language,$TimeZone)
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($email,$lang,$time) Set-MailboxRegionalConfiguration -Identity $email -Language $lang -TimeZone $time}-arg $UserEmail,$Language,$TimeZone}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully configure language ""$($Language)"" and timezone ""$($TimeZone)"" for ""$($UserDisplayName)"" ""($($UserEmail))"""
	log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Configuring language ""$($Language)"" and timezone ""$($TimeZone)"" for ""$($UserDisplayName)"" ""($($UserEmail))"". Error = $($error[0])"
    log_message "ERROR" $S
    }
}
#
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function exec
{
Param($UserDisplayName,$UserEmail,$Prefix,$LicensePacks,$UsageLocation)
$UserDisplayName=$UserDisplayName.Trim()
log_message "Office 365" "$($UserDisplayName) ($($UserEmail))"
if (user_exists($UserEmail))
   {
   display_user	$UserDisplayName $UserEmail
   if (UserMSolExist($UserEmail))
      {
      $LicensePackList=$LicensePacks.Split(",")
      ForEach ($LicensePack in $LicensePackList)
              {
              If (!(UserHasLicense $UserEmail $LicensePack))
                 {
                 $LicensePack=$Prefix+':'+$LicensePack
                 addlicensepack_user $UserDisplayName $UserEmail $LicensePack $UsageLocation
                 }
                 else
                 {
                 $global:errors_count++
                 $S="MSol User ""$($UserEmail)"" has already license pack : "+$LicensePack
                 ## $S=$S+(UserLicenses $UserEmail)
                 write-host $S -foregroundcolor "red"
                 log_message "ERROR" $S
                 }
            }      
      }
      else
      {
      $global:errors_count++
      $S="MSol User ""$($UserEmail)"" does not exists"
      write-host $S -foregroundcolor "red"
      log_message "ERROR" $S
      }
   regional_configuration_user $UserDisplayName $UserEmail $Language $TimeZone
   }
   else
   {
   $global:errors_count++
   $S="User ""$($UserEmail)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserEmail)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
}
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V2.81"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword="%OFFICE365_PASSWORD%"
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword="%PROXY_PASSWORD%"
#
# License pack parameters
#
$UsageLocation="FR"
#
# Exchange Online
#
$Language="fr-FR"
$TimeZone="Romance Standard Time"
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
$Credential=connect $Office365Url $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if (Install-MSOnlineCmdlets)
   {
   Connect-MSolService -Credential $Credential
   if ($global:total_errors_count -eq 0)
      {
      $Prefix=LicensePackDomain      
      ### BEGIN_REPETITION ###
      $global:errors_count=0
      $LicensePack="$($Prefix):%OFFICE365_LICENSE_PACK%"
      $UserDisplayName="%USER_TITLE% %USER_FIRST_NAME% %USER_LAST_NAME%"
      $UserEmail="%USER_EMAIL%"
      exec $UserDisplayName $UserEmail $LicensePack $UsageLocation
      ### END_REPETITION ###      
      }
   }
   else
   {
   $global:total_errors_count++	
   }	
 log_total_errors_count 
